"""
This module can load Python modules by path.
"""

from importlib import import_module
import inspect
import os
import pkgutil
import sys

from simple_plugin_loader.sample_plugin import SamplePlugin


class _Loader():
    def __init__(self):
        self.__available_plugins = {}

    @property
    def plugins(self):
        """
        Get all already loaded plugins.
        """
        return self.__available_plugins

    def load_plugins(self, path: str, plugin_base_class: type=SamplePlugin, recursive: bool=False) -> dict:
        """
        Load all classes in a directory specified by 'path' that match the 'plugin_base_class' class.

        All other classes or methods are ignored.
        """
        # normalize the path
        path = os.path.abspath(os.path.normpath(path))

        # add the parent path to the system PATH if it doesn't already exists
        # this is needed for the import later
        sys_path_modified = False
        if os.path.dirname(path) not in sys.path:
            # insert the parent path at index zero, because the loader should look at this location first
            sys.path.insert(0, os.path.dirname(path))
            sys_path_modified = True

        # do the actual import
        plugins = self.__load(path,
                              os.path.basename(path),  # the module main package is the last directory of the path
                              plugin_base_class,
                              recursive)

        # reset the modified path again
        if (sys_path_modified and
                os.path.dirname(path) in sys.path):
            sys.path.remove(os.path.dirname(path))

        self.__available_plugins.update(plugins)
        return plugins

    def __load(self, path: str, package_name: str, plugin_base_class: type=SamplePlugin, recursive: bool=False) -> dict:
        plugins = {}
        # iterate over the modules that are within the path
        for (_, name, ispkg) in pkgutil.iter_modules([path]):
            if ispkg:
                if recursive:
                    plugins.update(self.__load(os.path.join(path, name),
                                               ".".join([package_name, name]),
                                               plugin_base_class,
                                               recursive))
                    continue
                else:
                    # do not try to import it, since it's not a module
                    continue

            # import the module
            imported_module = import_module(".".join([package_name, name]))

            plugin_found = False
            # try to find a subclass of the plugin class
            for i in dir(imported_module):
                attribute = getattr(imported_module, i)

                if (inspect.isclass(attribute) and
                        # check for plugin subclass
                        issubclass(attribute, plugin_base_class) and
                        # but do not match the plugin class itself
                        attribute != plugin_base_class):
                    if hasattr(attribute, "plugin_name"):
                        # if the plugin is derived from 'SamplePlugin' class, use the 'plugin_name' property as name
                        pn = attribute.plugin_name
                    else:
                        # otherwise simply use the class name
                        pn = attribute.__name__

                    plugins[pn.casefold()] = attribute
                    plugin_found = True

            # remove imported module again if no plugin class is found
            if not plugin_found:
                del imported_module

        return plugins



from setuptools import setup
import setuptools


# gitpython is needed for version tagging
setuptools._install_setup_requires({"setup_requires": "gitpython"})
import git  # noqa


# get the description from README.md
with open("README.md", 'r') as readme:
    long_description = readme.read()


# get the version from git
def get_version():
    repo = git.Repo(search_parent_directories=True)

    if repo.tags:
        # get the latest tag
        latest_tag = sorted(
            repo.tags, key=lambda t: t.commit.committed_date, reverse=True)[0]
        # check if the latest commit is a release
        if repo.head.commit.hexsha == latest_tag.commit.hexsha:
            # just return the release tag
            return latest_tag.name
        else:
            # get amount of commits since the last tag as pre-release
            commit_count_since_latest_tag = len(
                list(repo.iter_commits(max_age=latest_tag.commit.authored_date))) - 1
            # return a pre-release tag
            return "%s.post%s" % (latest_tag.name, commit_count_since_latest_tag)
    else:
        # get the current commit count to use as pre-release
        git_commit_count = len(list(repo.iter_commits()))
        # default 'no-release' tag
        return "0.0a%s" % git_commit_count


# the actual setup
setup(install_requires=['simple-classproperty==1.2'],
      name="simple-plugin-loader",
      version=get_version(),
      author="Marc Ammon",
      author_email="marc.ammon@hotmail.de",
      description="Dynamically load other python modules to your project",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/mammo0/py-simple-plugin-loader",
      packages=["simple_plugin_loader"],
      classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Operating System :: OS Independent",
],
    python_requires='>=3.5',
)
